@@ -24,7 +24,7 @@
  *
  * Author(s): Berin Lautenbach
  *
- * $Id: OpenSSLCryptoKeyDSA.cpp 1843562 2018-10-11 15:13:40Z scantor $
+ * $Id$
  *
  */
 #include <xsec/framework/XSECDefs.hpp>
@@ -43,6 +43,23 @@ XSEC_USING_XERCES(ArrayJanitor);
 
 #include <openssl/dsa.h>
 
+// Inline for older OpenSSL versions.
+#if (OPENSSL_VERSION_NUMBER <   0x10100000L)
+static int BN_bn2binpad(const BIGNUM *a, unsigned char *to, int tolen)
+{
+    int bytes = BN_num_bytes(a);
+
+    if (bytes > tolen) {
+        return -1;
+    }
+    while (bytes < tolen) {
+        *to++ = 0;
+        bytes++;
+    }
+    BN_bn2bin(a, to);
+    return tolen;
+}
+#endif
 
 OpenSSLCryptoKeyDSA::OpenSSLCryptoKeyDSA() : mp_dsaKey(NULL), mp_accumP(NULL), mp_accumQ(NULL), mp_accumG(NULL) {
 };
@@ -395,25 +412,19 @@ unsigned int OpenSSLCryptoKeyDSA::signBase64Signature(unsigned char * hashBuf,
 
     DSA_SIG_get0(dsa_sig, &dsaSigR, &dsaSigS);
 
-    unsigned char* rawSigBuf = new unsigned char[(BN_num_bits(dsaSigR) + BN_num_bits(dsaSigS) + 7) / 8];
-    ArrayJanitor<unsigned char> j_sigbuf(rawSigBuf);
+    const int DSAsigCompLen = 20; // XMLDSIG spec 6.4.1
+    unsigned char rawSigBuf[2*DSAsigCompLen];
     
-    unsigned int rawLen = BN_bn2bin(dsaSigR, rawSigBuf);
-
-    if (rawLen <= 0) {
+    if (BN_bn2binpad(dsaSigR, rawSigBuf, DSAsigCompLen) <= 0) {
         throw XSECCryptoException(XSECCryptoException::DSAError,
             "OpenSSL:DSA - Error converting signature to raw buffer");
     }
 
-    unsigned int rawLenS = BN_bn2bin(dsaSigS, (unsigned char *) &rawSigBuf[rawLen]);
-
-    if (rawLenS <= 0) {
+    if (BN_bn2binpad(dsaSigS, rawSigBuf+DSAsigCompLen, DSAsigCompLen) <= 0) {
         throw XSECCryptoException(XSECCryptoException::DSAError,
             "OpenSSL:DSA - Error converting signature to raw buffer");
     }
 
-    rawLen += rawLenS;
-
     // Now convert to Base 64
 
     BIO * b64 = BIO_new(BIO_f_base64());
@@ -424,7 +435,7 @@ unsigned int OpenSSLCryptoKeyDSA::signBase64Signature(unsigned char * hashBuf,
 
     // Translate signature from Base64
 
-    BIO_write(b64, rawSigBuf, rawLen);
+    BIO_write(b64, rawSigBuf, 2*DSAsigCompLen);
     BIO_flush(b64);
 
     unsigned int sigValLen = BIO_read(bmem, base64SignatureBuf, base64SignatureBufLen);
@@ -437,11 +448,9 @@ unsigned int OpenSSLCryptoKeyDSA::signBase64Signature(unsigned char * hashBuf,
     }
 
     return sigValLen;
-
 }
 
 
-
 XSECCryptoKey * OpenSSLCryptoKeyDSA::clone() const {
 
     OpenSSLCryptoKeyDSA * ret;
